<?php

namespace FiloBlu\OrderStatus\Observer;

use \Magento\Framework\Event\Observer;
use \Magento\Framework\Event\ObserverInterface;
use \Magento\Store\Model\ScopeInterface;
use \Magento\Framework\App\Config\ScopeConfigInterface;
use \Psr\Log\LoggerInterface;

class UpdateStatusAmazonPayIpn implements ObserverInterface
{
    /**
     * @var LoggerInterface
     */
    protected $_logger;

    /**
     * @var ScopeConfigInterface
     */
    protected $_config;

    const IPN_COMPLETED_ORDER_STATUS = 'payment_confirmed';
    const IPN_COMPLETED_ORDER_STATUS_CONFIG_PATH = 'payment/adyen_abstract/payment_authorized';
    const CONFIRMED_ORDER_STATUS = 'confirmed';
    const CONFIRMED_ORDER_STATUS_CONFIG_PATH = 'payment/adyen_abstract/payment_pre_authorized';
    const ORDER_EXPORTED_ORDER_STATUS = 'order_exported';
    const ORDER_EXPORTED_ORDER_STATUS_CONFIG_PATH = 'filoblu_paymentscapture/general/order_status';
    const PROCESSING_ORDER_STATUS = 'processing';
    const DEBUG_LOG_ACTIVE = TRUE;

    /**
     * @param LoggerInterface $logger
     * @param ScopeConfigInterface $config
     */
    public function __construct(
        LoggerInterface $logger,
        ScopeConfigInterface $config
    ) {
        $this->_logger = $logger;
        $this->_config = $config;
    }

    public function execute(Observer $observer)
    {
        $order = $observer->getEvent()->getOrder();
        $action = $observer->getEvent()->getAction();

        $captured_order_status = $this->_config->getValue(self::IPN_COMPLETED_ORDER_STATUS_CONFIG_PATH, ScopeInterface::SCOPE_STORE) ?
            $this->_config->getValue(self::IPN_COMPLETED_ORDER_STATUS_CONFIG_PATH, ScopeInterface::SCOPE_STORE) : self::IPN_COMPLETED_ORDER_STATUS;

        $confirmed_order_status = $this->_config->getValue(self::CONFIRMED_ORDER_STATUS_CONFIG_PATH, ScopeInterface::SCOPE_STORE) ?
            $this->_config->getValue(self::CONFIRMED_ORDER_STATUS_CONFIG_PATH, ScopeInterface::SCOPE_STORE) : self::CONFIRMED_ORDER_STATUS;

        $order_exported_order_status = $this->_config->getValue(self::ORDER_EXPORTED_ORDER_STATUS_CONFIG_PATH, ScopeInterface::SCOPE_STORE) ?
            $this->_config->getValue(self::ORDER_EXPORTED_ORDER_STATUS_CONFIG_PATH, ScopeInterface::SCOPE_STORE) : self::ORDER_EXPORTED_ORDER_STATUS;

        $validStatusToChangeOrder = array($confirmed_order_status, $order_exported_order_status, self::PROCESSING_ORDER_STATUS);

        $orderStatus = $order->getStatus();

        switch ($action) {
            case 'captured':
                if($orderStatus != $captured_order_status && in_array($orderStatus, $validStatusToChangeOrder))
                {
                    $order->addStatusHistoryComment(
                        __('Order processed by mod2-order-status'), $captured_order_status
                    );
                    $order->save();

                    if (self::DEBUG_LOG_ACTIVE) {
                        $this->_logger->info("FiloBlu\UpdateStatusAmazonPayIpn\Observer: Order status has been set to " . $captured_order_status);
                    }
                }
                break;
        }

        return TRUE;
    }
}